// Minimalist Canvas manager
export default class {

  constructor({id = 'canvas', draw = () => 0, responsive = true, width = 1000, height = 600, onclick = () => 0} = {}) {
    this.ctx = document.getElementById(id).getContext('2d');
    this.responsive = responsive;
    if (responsive) {
      this.ctx.canvas.width = this.ctx.canvas.clientWidth;
      this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    } else {
      this.ctx.canvas.width = width;
      this.ctx.canvas.height = height;
    }
    this.callbackDraw = draw;
    this.ctx.canvas.onclick = onclick;
  }
  
  onclick(onclick) {
    this.ctx.canvas.onclick = onclick;
  }

  getDimension() {
    return {width: this.ctx.canvas.width, height: this.ctx.canvas.height};
  }

  isInLandscape() {
    return this.ctx.canvas.width > this.ctx.canvas.height;
  }

  isInPortrait() {
    return !this.isInLandscape();
  }

  get() {
    return this.ctx.canvas;
  }

  getCtx() {
    return this.ctx;
  }

  draw() {
    this.callbackDraw(this.ctx);
  }

  redraw() {
    if (this.responsive) {
      this.ctx.canvas.width = this.ctx.canvas.clientWidth;
      this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    } else {
      this.ctx.clearRect(0, 0, this.ctx.canvas.width, this.ctx.canvas.height);
    }
    this.draw();
  }

}
