// Minimalist game loop
export default class {

  constructor({panic = true, panicThreshold = 1000/15, panicCallback = () => false, update = t => true, draw = t => true} = {}) {
    this.loop = null;
    this.panic = panic;
    this.panicThreshold = panicThreshold;
    this.panicCallback = panicCallback;
    this.update = update;
    this.draw = draw;
    this.elapsedTime = 0;
  }

  start() {
    this.loop = requestAnimationFrame(now => {
      this.lastTickTime = now;
      this._tick(now);
    });
  }

  stop() {
    cancelAnimationFrame(this.loop);
    this.loop = null;
  }

  toggle() {
    this.isRunning() ? this.stop() : this.start();
  }

  isRunning() {
    return this.loop != null;
  }

  getElapsedTime() {
    return this.elapsedTime;
  }

  _tick(now) {
    this.loop = requestAnimationFrame(now => this._tick(now));
    let deltaTime = now - this.lastTickTime;
    this.lastTickTime = now;
    if (this.panic && deltaTime > this.panicThreshold && !this.panicCallback()) return;
    this.elapsedTime += deltaTime;
    this.update(deltaTime);
    this.draw(deltaTime);
  }

}