import WebSocket from "ws";

export default class {

  constructor({port = 8998, clientsPerRoom = 4} = {}) {
    console.log(`WebSocket server on port ${port} is waiting for connection`);
    this.clientsPerRoom = clientsPerRoom;
    this.rooms = new Map();
    this.clientId = 1;
    this.wss = new WebSocket.Server({port});
    this.wss.on('connection', client => {
      const room = this.findOrCreateRoom();
      room.set(client, this.clientId++);
      // todo: json message validation
      client.on('message', message => this.broadcast(room, message, client));
      client.on('close', () => {
        this.broadcast(room, {action: 'drop'}, client, false);
        let cid = room.get(client);
        room.delete(client);
        if (room.size == 0) {
          this.rooms.delete(room);
          return;
        }
        // Send a "giveOwnership" message to someone still connected to handle
        // AFRAME ws-synch component in "unique entity" mode
        let newOwner = room.keys().next().value;
        this.send(room, {action: 'giveOwnership'}, cid, newOwner, false);
      });
    });
  }

  send(room, message, cid, toClient, isJson = true) {
    if (isJson) message = JSON.parse(message);
    let data = {message, cid};
    toClient.send(JSON.stringify(data));
  }

  broadcast(room, message, itself, isJson = true) {
    let cid = room.get(itself);
    if (isJson) message = JSON.parse(message);
    let data = {message, cid};
    for (let client of room.keys()) {
      if (client !== itself && client.readyState === WebSocket.OPEN) {
        client.send(JSON.stringify(data));
      }
    }
  }

  findOrCreateRoom() {
    let lessPopulatedRoom = null;
    let nbClientsInLessPopulateRoom = this.clientsPerRoom;
    for (let room of this.rooms.keys()) {
      if (room.size < nbClientsInLessPopulateRoom) {
        nbClientsInLessPopulateRoom = room.size;
        lessPopulatedRoom = room;
      }
    }
    if (lessPopulatedRoom === null) {
      lessPopulatedRoom = new Map();
      this.rooms.set(lessPopulatedRoom, lessPopulatedRoom);
    }
    return lessPopulatedRoom;
  }

}